@@ -3,14 +3,13 @@ import threading
 import time
 import traceback
 from concurrent.futures import ThreadPoolExecutor
-from typing import List
 
 import ophyd
 from bec_lib import Alarms, BECService, MessageEndpoints, bec_logger, messages
 from bec_lib.connector import ConnectorBase
-from bec_lib.devicemanager import OnFailure
+from bec_lib.device import OnFailure
 from bec_lib.messages import BECStatus
-from ophyd import Staged
+from ophyd import OphydObject, Staged
 from ophyd.utils import errors as ophyd_errors
 
 from device_server.devices.devicemanager import DeviceManagerDS
@@ -50,7 +49,7 @@ class DeviceServer(RPCMixin, BECService):
         self._start_device_manager()
 
     def _start_device_manager(self):
-        self.device_manager = DeviceManagerDS(self.connector, status_cb=self.update_status)
+        self.device_manager = DeviceManagerDS(self, status_cb=self.update_status)
         self.device_manager.initialize(self.bootstrap_server)
 
     def start(self) -> None:
@@ -64,7 +63,7 @@ class DeviceServer(RPCMixin, BECService):
                 event=consumer_stop,
                 cb=self.instructions_callback,
                 parent=self,
-            ),
+            )
         ]
         for thread in self.threads:
             thread.start()
@@ -94,7 +93,8 @@ class DeviceServer(RPCMixin, BECService):
         if not isinstance(devices, list):
             devices = [devices]
         for dev in devices:
-            self.device_manager.devices.get(dev).metadata = instr.metadata
+            device_root = dev.split(".")[0]
+            self.device_manager.devices.get(device_root).metadata = instr.metadata
 
     @staticmethod
     def consumer_interception_callback(msg, *, parent, **_kwargs) -> None:
@@ -104,7 +104,7 @@ class DeviceServer(RPCMixin, BECService):
             logger.warning("Failed to parse scan queue modification message.")
             return
         logger.info(f"Receiving: {mvalue.content}")
-        if mvalue.content.get("action") in ["pause", "abort"]:
+        if mvalue.content.get("action") in ["pause", "abort", "halt"]:
             parent.stop_devices()
 
     def stop_devices(self) -> None:
@@ -112,7 +112,7 @@ class DeviceServer(RPCMixin, BECService):
         logger.info("Stopping devices after receiving 'abort' request.")
         self.status = BECStatus.BUSY
         for dev in self.device_manager.devices.enabled_devices:
-            if not dev.enabled_set:
+            if dev.read_only:
                 # don't stop devices that we haven't set
                 continue
             if hasattr(dev.obj, "stop"):
@@ -126,6 +126,7 @@ class DeviceServer(RPCMixin, BECService):
             devices = [devices]
 
         for dev in devices:
+            dev = dev.split(".")[0]
             if not self.device_manager.devices[dev].enabled:
                 raise DisabledDeviceError(f"Cannot access disabled device {dev}.")
 
@@ -136,6 +137,7 @@ class DeviceServer(RPCMixin, BECService):
         if isinstance(devices, str):
             devices = [devices]
         for dev in devices:
+            dev = dev.split(".")[0]
             if dev not in self.device_manager.devices:
                 raise InvalidDeviceError(f"There is no device with the name {dev}.")
 
@@ -256,7 +258,7 @@ class DeviceServer(RPCMixin, BECService):
 
     def _set_device(self, instr: messages.DeviceInstructionMessage) -> None:
         device_obj = self.device_manager.devices.get(instr.content["device"])
-        if not device_obj.enabled_set:
+        if device_obj.read_only:
             raise DisabledDeviceError(
                 f"Setting the device {device_obj.name} is currently disabled."
             )
@@ -279,9 +281,7 @@ class DeviceServer(RPCMixin, BECService):
             if hasattr(obj.obj, "pre_scan"):
                 obj.obj.pre_scan()
             dev_msg = messages.DeviceReqStatusMessage(
-                device=dev,
-                success=True,
-                metadata=instr.metadata,
+                device=dev, success=True, metadata=instr.metadata
             ).dumps()
             self.producer.set_and_publish(MessageEndpoints.device_req_status(dev), dev_msg, pipe)
         pipe.execute()
@@ -293,9 +293,7 @@ class DeviceServer(RPCMixin, BECService):
         else:
             device_name = status.obj.root.name
         dev_msg = messages.DeviceReqStatusMessage(
-            device=device_name,
-            success=status.success,
-            metadata=status.instruction.metadata,
+            device=device_name, success=status.success, metadata=status.instruction.metadata
         ).dumps()
         logger.debug(f"req status for device {device_name}: {status.success}")
         self.producer.set_and_publish(
@@ -320,54 +318,35 @@ class DeviceServer(RPCMixin, BECService):
 
         self._read_and_update_devices(devices, instr.metadata)
 
-    def _read_and_update_devices(self, devices: List[str], metadata: dict) -> list:
+    def _read_and_update_devices(self, devices: list[str], metadata: dict) -> list:
         start = time.time()
         pipe = self.producer.pipeline()
         signal_container = []
         for dev in devices:
-            self.device_manager.devices.get(dev).metadata = metadata
-            obj = self.device_manager.devices.get(dev).obj
+            device_root = dev.split(".")[0]
+            self.device_manager.devices.get(device_root).metadata = metadata
+            obj = self.device_manager.devices.get(device_root).obj
             try:
                 signals = obj.read()
                 signal_container.append(signals)
             except Exception as exc:
-                self.device_manager.connector.raise_alarm(
-                    severity=Alarms.WARNING,
-                    alarm_type="Warning",
-                    source="DeviceServer",
-                    content=f"Failed to read device {dev}.",
-                    metadata={},
-                )
-                ds_dev = self.device_manager.devices.get(dev)
-                if ds_dev.on_failure == OnFailure.RAISE:
-                    raise exc
-
-                if ds_dev.on_failure == OnFailure.RETRY:
-                    # try to read it again, may have been only a glitch
-                    signals = obj.read()
-                elif ds_dev.on_failure == OnFailure.BUFFER:
-                    # if possible, fall back to past readings
-                    logger.warning(f"Failed to read {dev}. Trying to load an old value.")
-                    old_msg = messages.DeviceMessage.loads(
-                        self.producer.get(MessageEndpoints.device_read(dev))
-                    )
-                    if not old_msg:
-                        raise exc
-                    signals = old_msg.content["signals"]
+                signals = self._retry_obj_method(dev, obj, "read", exc)
 
             self.producer.set_and_publish(
-                MessageEndpoints.device_read(dev),
+                MessageEndpoints.device_read(device_root),
                 messages.DeviceMessage(signals=signals, metadata=metadata).dumps(),
                 pipe,
             )
             self.producer.set_and_publish(
-                MessageEndpoints.device_readback(dev),
+                MessageEndpoints.device_readback(device_root),
                 messages.DeviceMessage(signals=signals, metadata=metadata).dumps(),
                 pipe,
             )
             self.producer.set(
-                MessageEndpoints.device_status(dev),
-                messages.DeviceStatusMessage(device=dev, status=0, metadata=metadata).dumps(),
+                MessageEndpoints.device_status(device_root),
+                messages.DeviceStatusMessage(
+                    device=device_root, status=0, metadata=metadata
+                ).dumps(),
                 pipe,
             )
         pipe.execute()
@@ -376,6 +355,65 @@ class DeviceServer(RPCMixin, BECService):
         )
         return signal_container
 
+    def _read_config_and_update_devices(self, devices: list[str], metadata: dict) -> list:
+        start = time.time()
+        pipe = self.producer.pipeline()
+        signal_container = []
+        for dev in devices:
+            self.device_manager.devices.get(dev).metadata = metadata
+            obj = self.device_manager.devices.get(dev).obj
+            try:
+                signals = obj.read_configuration()
+                signal_container.append(signals)
+            except Exception as exc:
+                signals = self._retry_obj_method(dev, obj, "read_configuration", exc)
+            self.producer.set_and_publish(
+                MessageEndpoints.device_read_configuration(dev),
+                messages.DeviceMessage(signals=signals, metadata=metadata).dumps(),
+                pipe,
+            )
+        pipe.execute()
+        logger.debug(
+            f"Elapsed time for reading and updating status info: {(time.time()-start)*1000} ms"
+        )
+        return signal_container
+
+    def _retry_obj_method(self, device: str, obj: OphydObject, method: str, exc: Exception) -> dict:
+        self.device_manager.connector.raise_alarm(
+            severity=Alarms.WARNING,
+            alarm_type="Warning",
+            source="DeviceServer",
+            content=f"Failed to run {method} on device {device}.",
+            metadata={},
+        )
+        device_root = device.split(".")[0]
+        ds_dev = self.device_manager.devices.get(device_root)
+        if ds_dev.on_failure == OnFailure.RAISE:
+            raise exc
+
+        if ds_dev.on_failure == OnFailure.RETRY:
+            # try to read it again, may have been only a glitch
+            signals = getattr(obj, method)()
+        elif ds_dev.on_failure == OnFailure.BUFFER:
+            # if possible, fall back to past readings
+            logger.warning(
+                f"Failed to run {method} on device {device_root}. Trying to load an old value."
+            )
+            if method == "read":
+                old_msg = messages.DeviceMessage.loads(
+                    self.producer.get(MessageEndpoints.device_read(device_root))
+                )
+            elif method == "read_configuration":
+                old_msg = messages.DeviceMessage.loads(
+                    self.producer.get(MessageEndpoints.device_read_configuration(device_root))
+                )
+            else:
+                raise ValueError(f"Unknown method {method}.")
+            if not old_msg:
+                raise exc
+            signals = old_msg.content["signals"]
+        return signals
+
     def _stage_device(self, instr: messages.DeviceInstructionMessage) -> None:
         devices = instr.content["device"]
         if not isinstance(devices, list):
